<?php


namespace app\cbcrm\model\customer;


use app\cbcrm\model\config\AreaModel;
use app\cbcrm\model\config\DictionaryModel;
use app\cbcrm\model\member\MemberModel;
use think\Exception;
use think\facade\Db;
use think\Model;
use think\model\concern\SoftDelete;

class CustomerModel extends Model
{
    use SoftDelete;
    protected $table = 'cb_crm_customer';

    public function getList($where, $wid)
    {
        $list = $this->withJoin(['contacts','area', 'user'],'LEFT')->where('contacts.default',1)->where($where)->where('customer_model.wid', $wid)->order('customer_model.create_time', 'desc')->paginate(15);

        foreach ($list as $row) {
            if ($row['area']) {
                $areaStr = str_replace(',', '', $row['area']['mergename']);
                $row['areaData'] = str_replace('中国', '', $areaStr);
            }
            if (!$row['is_super']) {
                $row['staff'] = Db::table('cb_crm_staff')->where('wid',$wid)->where('id',$row['staff_id'])->field('id,nickname')->find();
            }else{
                $row['staff'] = null;
            }
            $row->customerStatus;
        }
        return $list;
    }

    /*
     * 添加客户
     * */
    public function addCustomer($params, $wid)
    {
        $result = false;
        Db::startTrans();
        try {
            $params['wid'] = $wid;
            $check = $this->checkRepeatCustomer($params['name'], $wid);
            if ($check) {
                throw new Exception('客户已存在，请检查客户名称并查重');
            }
            /*
             * 会员参数
             * */
            if (isset($params['user_username']) && isset($params['user_password'])) {
                if ($this->checkUser('username', $params['user_username'])) {
                    throw new Exception('会员登录账号已存在');
                }
                if (isset($params['user_mobile'])) {
                    if (!preg_match("/^1[23456789]\d{9}$/", $params['user_mobile'])) {
                        throw new Exception('手机号格式错误，请重新输入');
                    }
                    if ($this->checkUser('mobile', $params['user_mobile'])) {
                        throw new Exception('会员手机号已存在');
                    }
                };
                $salt = random_build('alnum', 5);
                $userParmas = [
                    'username' => $params['user_username'],
                    'mobile' => $params['user_mobile'],
                    'password' => md5(md5($params['user_password']) . $salt),
                    'salt' => $salt,
                    'wid' => $wid,
                    'create_time' => isset($params['create_time']) ? $params['create_time'] : date('Y-m-d H:i:s'),
                    'update_time' => isset($params['create_time']) ? $params['create_time'] : date('Y-m-d H:i:s'),
                ];
                $user_id = Db::table('cb_crm_user')->insertGetId($userParmas);
                $params['user_id'] = $user_id ? $user_id : null;
            }
            $this->save($params);

            // 记录日志
            (new OperateModel())->addLog('添加客户',$this->id,$wid,$params);
            /*
             * 联系人参数
             * */
            $contactsParmas = [
                'wid' => $wid,
                'customer_id' => $this->id,
                'name' => isset($params['contact_name']) ? $params['contact_name'] : '',
                'gender' => isset($params['contact_gender']) ? $params['contact_gender'] : '',
                'tel' => isset($params['contact_tel']) ? $params['contact_tel'] : '',
                'mobile' => isset($params['contact_mobile']) ? $params['contact_mobile'] : '',
                'wechat' => isset($params['contact_wechat']) ? $params['contact_wechat'] : '',
                'qq' => isset($params['contact_qq']) ? $params['contact_qq'] : '',
                'position' => isset($params['contact_position']) ? $params['contact_position'] : '',
                'default' => 1,
                'role_id' => 84,
                'staff_id' => $params['staff_id'],
                'is_super' => $params['is_super'],
                'create_time' => isset($params['create_time']) ? $params['create_time'] : date('Y-m-d H:i:s'),
                'update_time' => isset($params['create_time']) ? $params['create_time'] : date('Y-m-d H:i:s'),
            ];
            $result = Db::table('cb_crm_contacts')->insert($contactsParmas);
            // 记录日志
            (new OperateModel())->addLog("添加客户联系人[{$contactsParmas['name']}]",$this->id,$wid,$contactsParmas);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 注册会员 添加数据
     * */
    public function registerCustomer($params, $wid)
    {
        /*
         * 客户参数
         * */
        $customerParams = [
            'wid' => $wid,
            'user_id' => $params['user_id'],
            'name' => $params['nickname'],
            'tel' => $params['mobile'],
            'level_id' => 28,
            'category_id' => 21,
            'source_id' => 86,
            'status_id' => 60,
            'is_super' => isset($params['staff_id'])?0:1,
            'staff_id' => isset($params['staff_id'])?$params['staff_id']:$wid,
            'entrytime' => date('Y-m-d H:i:s'),
            'create_time' => date('Y-m-d H:i:s'),
            'update_time' => date('Y-m-d H:i:s'),
        ];
        $this->save($customerParams);
        /*
         * 联系人参数
         * */
        $contactsParmas = [
            'wid' => $wid,
            'customer_id' => $this->id,
            'name' => $params['nickname'],
            'tel' => $params['mobile'],
            'mobile' => $params['mobile'],
            'default' => 1,
            'role_id' => 84,
            'is_super' => isset($params['staff_id'])?0:1,
            'staff_id' => isset($params['staff_id'])?$params['staff_id']:$wid,
            'create_time' => date('Y-m-d H:i:s'),
            'update_time' => date('Y-m-d H:i:s'),
        ];
        Db::table('cb_crm_contacts')->insert($contactsParmas);

    }


    /*
     * 编辑客户
     * */
    public function editCustomer($id, $params, $wid)
    {
        $customerInfo = $this->getDetail($id, $wid);
        if (!$customerInfo['customer']) {
            return '客户不存在';
        }
        $result = false;
        Db::startTrans();
        try {
            $check = $this->checkRepeatCustomer($params['name'], $wid, $id);
            if ($check) {
                throw new Exception('客户已存在，请检查客户名称并查重');
            }

            // 存在客户会员则编辑，否则新增会员
            if ($customerInfo['user']) {
                if (isset($params['user_username']) && !$params['user_username']) {
                    $params['user_id'] = null; // 解除会员绑定，不删除会员
                }
                if (isset($params['user_username']) && $params['user_username']) {
                    if ($this->checkUser('username', $params['user_username'], $customerInfo['user']['id'])) {
                        throw new Exception('会员登录账号已存在');
                    }
                }

                if (isset($params['user_mobile']) && $params['user_mobile']) {
                    if (!preg_match("/^1[23456789]\d{9}$/", $params['user_mobile'])) {
                        throw new Exception('手机号格式错误，请重新输入');
                    }
                    if ($this->checkUser('mobile', $params['user_mobile'], $customerInfo['user']['id'])) {
                        throw new Exception('会员手机号已存在');
                    }
                };

                $userParmas = [
                    'username' => $params['user_username'],
                    'mobile' => $params['user_mobile'],
                    'update_time' => date('Y-m-d H:i:s'),
                ];
                if (isset($params['user_password']) && $params['user_password']) {
                    // 密码不为空则修改密码
                    $salt = random_build('alnum', 5);
                    $userParmas['password'] = md5(md5($params['user_password']) . $salt);
                    $userParmas['salt'] = $salt;
                }
                Db::table('cb_crm_user')->where('id', $customerInfo['user']['id'])->update($userParmas);

            } else {
                if (isset($params['user_username']) && isset($params['user_password'])) {
                    if ($this->checkUser('username', $params['user_username'])) {
                        throw new Exception('会员登录账号已存在');
                    }
                    if (isset($params['user_mobile']) && $params['user_mobile']) {
                        if (!preg_match("/^1[23456789]\d{9}$/", $params['user_mobile'])) {
                            throw new Exception('手机号格式错误，请重新输入');
                        }
                        if ($this->checkUser('mobile', $params['user_mobile'])) {
                            throw new Exception('会员手机号已存在');
                        }
                    };
                    $salt = random_build('alnum', 5);
                    $userParmas = [
                        'username' => $params['user_username'],
                        'mobile' => $params['user_mobile'],
                        'password' => md5(md5($params['user_password']) . $salt),
                        'salt' => $salt,
                        'wid' => $wid,
                        'create_time' => date('Y-m-d H:i:s'),
                        'update_time' => date('Y-m-d H:i:s'),
                    ];
                    $user_id = Db::table('cb_crm_user')->insertGetId($userParmas);
                    $params['user_id'] = $user_id ? $user_id : null;
                }
            }
            $params['update_time'] = date('Y-m-d H:i:s');
            $result = $this->where('id', $id)->update($params);
            // 记录日志
            (new OperateModel())->addLog('编辑客户',$id,$wid,$params);
            /*
             * 联系人参数
             * */
            $contactsParmas = [
                'name' => isset($params['contact_name']) ? $params['contact_name'] : '',
                'gender' => isset($params['contact_gender']) ? $params['contact_gender'] : '',
                'tel' => isset($params['contact_tel']) ? $params['contact_tel'] : '',
                'mobile' => isset($params['contact_mobile']) ? $params['contact_mobile'] : '',
                'wechat' => isset($params['contact_wechat']) ? $params['contact_wechat'] : '',
                'qq' => isset($params['contact_qq']) ? $params['contact_qq'] : '',
                'position' => isset($params['contact_position']) ? $params['contact_position'] : '',
                'default' => 1,
                'staff_id' => $params['staff_id'],
                'is_super' => $params['is_super'],
                'update_time' => date('Y-m-d H:i:s'),
            ];
            Db::table('cb_crm_contacts')->where('id', $customerInfo['contact']['id'])->update($contactsParmas);
            // 记录日志
            (new OperateModel())->addLog("编辑客户联系人[{$contactsParmas['name']}]",$id,$wid,$contactsParmas);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 删除客户
     * */
    public function delCustomer($id, $wid)
    {
        $row = $this->where('id', $id)->where('wid', $wid)->find();
        $result = $row->delete();
        // 记录日志
        (new OperateModel())->addLog('删除客户',$id,$wid);
        if ($result) {
            return true;
        } else {
            return '删除失败';
        }
    }

    /*
     * 获取客户信息
     * */
    public function getDetail($id, $wid)
    {
        $customer = $this->with(['customer_status', 'customer_type', 'customer_level', 'area'])->where('id', $id)->where('wid', $wid)->find();
        if (!$customer) {
            abort(404, '客户不存在');
        }
        if ($customer->label_ids) {
            $label_ids = explode(',', $customer->label_ids);
            foreach ($label_ids as &$id) {
                $id = (int)$id;
            }
            $customer->label_ids = $label_ids;
        }else{
            $customer->label_ids = [];
        }
        $contact = null; // 默认联系人
        $tax_contact = null; // 报税联系人
        $contactList = Db::table('cb_crm_contacts')->where('customer_id', $customer->id)->select();
        foreach ($contactList as $item) {
            if ($item['default'] == 1) {
                $contact = $item;
            }
            if ($item['role_id'] == 85) {
                $tax_contact = $item;
            }
        }
        $user = (new MemberModel())->with(['grade'])->where('id', $customer->user_id)->find();
        $customer['areaData'] = '';
        if ($customer['area']) {
            $areaStr = str_replace(',', '', $customer['area']['mergename']);
            $customer['areaData'] = str_replace('中国', '', $areaStr);
        }

        if (!$customer['is_super']) {
            $staff = Db::table('cb_crm_staff')->where('wid',$wid)->where('id',$customer['staff_id'])->field('id,nickname')->find();
        }else{
            $staff = null;
        }

        return compact('customer', 'contact', 'user', 'tax_contact','staff');
    }

    public function getFormData($wid)
    {

        $customerLabel = $customerStatus = $customerLevel = $customerSource = $customerType = $customerIndustry = $contactPosition = $registerType = $taxType = $taxReturn = $taxTray = $taxTrayType = $taxList = $incomeTax = [];
        $list = (new DictionaryModel())->getTypeValues([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15], $wid);
        foreach ($list as $item) {
            switch ($item['type']) {
                case 1:
                    // 客户种类
                    array_push($customerType, $item);
                    break;
                case 2:
                    // 客户级别
                    array_push($customerLevel, $item);
                    break;
                case 3:
                    // 客户来源
                    array_push($customerSource, $item);
                    break;
                case 4:
                    // 客户行业
                    array_push($customerIndustry, $item);
                    break;
                case 5:
                    // 成交状态
                    array_push($customerStatus, $item);
                    break;
                case 6:
                    // 客户类型
                    array_push($customerLabel, $item);
                    break;
                case 7:
                    // 企业登记类型
                    array_push($registerType, $item);
                    break;
                case 8:
                    // 主管税务机关
                    array_push($taxList, $item);
                    break;
                case 9:
                    // 纳税类型
                    array_push($taxType, $item);
                    break;
                case 10:
                    // 报税类型
                    array_push($taxReturn, $item);
                    break;
                case 11:
                    // 税盘存放
                    array_push($taxTray, $item);
                    break;
                case 12:
                    // 税盘类型
                    array_push($taxTrayType, $item);
                    break;
                case 13:
                    // 所得税类型
                    array_push($incomeTax, $item);
                    break;
                case 14:
                    // 联系人职务
                    array_push($contactPosition, $item);
                    break;
            }
        }
        return compact('customerLabel', 'customerStatus', 'customerLevel', 'customerSource', 'customerType', 'customerIndustry', 'contactPosition', 'registerType', 'taxType', 'taxReturn', 'taxTray', 'taxTrayType', 'taxList', 'incomeTax');

    }

    /*
     * 检测客户是否重复
     * */
    public function checkRepeatCustomer($name, $wid, $id = 0)
    {
        $query = $this->where('name', $name)->where('wid', $wid);
        if ($id) {
            $query->where('id', '<>', $id);
        }
        return $query->count();
    }

    /*
     * 检测会员账号等是否重复
     * */
    public function checkUser($field, $account, $id = 0)
    {
        $query = Db::table('cb_crm_user')->where($field, $account);
        if ($id) {
            $query->where('id', '<>', $id);
        }
        return $query->count();
    }

    /*
     * 联系人关联
     * */
    public function contacts()
    {
        return $this->belongsTo(ContactsModel::class, 'id', 'customer_id');
    }

    public function customerStatus()
    {
        return $this->belongsTo(DictionaryModel::class, 'status_id', 'id')->field('id,name');
    }

    public function customerType()
    {
        return $this->belongsTo(DictionaryModel::class, 'category_id', 'id')->field('id,name');
    }

    public function customerLevel()
    {
        return $this->belongsTo(DictionaryModel::class, 'level_id', 'id')->field('id,name');
    }

    public function user()
    {
        return $this->belongsTo(MemberModel::class, 'user_id', 'id');
    }

    public function area()
    {
        return $this->belongsTo(AreaModel::class, 'county_id', 'id')->field('id,shortname,name,mergename');
    }
}
