<?php


namespace app\cbcrm\model\customer;


use app\cbcrm\model\config\DictionaryModel;
use app\cbcrm\model\manage\StaffModel;
use think\Exception;
use think\facade\Db;
use think\facade\Log;
use think\Model;
use think\model\concern\SoftDelete;

class FollowModel extends Model
{
    use SoftDelete;

    protected $table = 'cb_crm_follow';

    protected $append = [
        'follow_mode_text'
    ];

    public function getList($where,$wid)
    {
       $list = $this->withJoin(['customer','contact'],'LEFT')
           ->where($where)
           ->order('create_time','desc')
           ->paginate(10);
        foreach ($list as $row) {
            if (!$row['is_super']) {
                $row['staff'] = Db::table('cb_crm_staff')->where('wid',$wid)->where('id',$row['staff_id'])->field('id,nickname')->find();
            }else{
                $row['staff'] = null;
            }
            $filesArr = [];
            if ($row->files) {
                $files = explode(',',$row->files);
                foreach ($files as $file) {
                    array_push($filesArr,[
                        'url'=>$file,
                        'preview_url'=>cdnurl($file)
                    ]);
                }
            }
            $row->filesArr = $filesArr;
        }

       return $list;
    }

    /*
     * 添加跟单记录
     * */
    public function addFollow($params,$wid)
    {
        $result = false;
        Db::startTrans();
        try {
            $params['wid'] = $wid;
            if ($params['files']) {
                $params['files'] = implode(',',$params['files']);
            }
            $result = $this->save($params);
            (new OperateModel())->addLog("添加客户跟单记录",$params['customer_id'],$wid,$params);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 编辑跟单记录
     * */
    public function editFollow($id,$params,$wid)
    {
        $row = $this->where('id',$id)->where('wid',$wid)->find();
        if (!$row) {
            abort(404,'记录不存在');
        }
        $result = false;
        Db::startTrans();
        try {
            if ($params['files']) {
                $params['files'] = implode(',',$params['files']);
            }
            $result = $row->save($params);
            (new OperateModel())->addLog("编辑客户跟单",$params['customer_id'],$wid,$params);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 删除跟单记录
     * */
    public function delFollow($id,$wid)
    {
        $row = $this->where('id',$id)->where('wid',$wid)->find();
        if (!$row) {
            abort(404,'记录不存在');
        }
        $result = false;
        Db::startTrans();
        try {
            $result = $row->delete();
            (new OperateModel())->addLog('删除客户跟单',$row['customer_id'],$wid,$row);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    public function getFormData($wid)
    {
        $followModeList = (new DictionaryModel())->getDicValues('跟单方式', $wid);

        return compact('followModeList');
    }

    public function getFollowModeTextAttr($data,$row){
        $dictionary = (new DictionaryModel())->getDicValues('跟单方式', $row['id']);
        foreach ($dictionary as $item) {
            if ($row['follow_mode'] == $item['id']) {
                return $item['name'];
            }
        }
        return '';
    }

    public function customer()
    {
        return $this->belongsTo(CustomerModel::class,'customer_id','id')->field('id,name');
    }

    public function contact()
    {
        return $this->belongsTo(ContactsModel::class, 'contact_id', 'id');
    }
}
