<?php


namespace app\cbcrm\model\customer;


use app\cbcrm\model\config\DictionaryModel;
use think\Exception;
use think\facade\Db;
use think\Model;
use think\model\concern\SoftDelete;

class NicheModel extends Model
{
    use SoftDelete;

    protected $table = 'cb_crm_niche';

    protected $append = [
        'stage_text',
        'status_text'
    ];

    public function getList($where,$wid)
    {
        $list = $this->withJoin(['customer','contact'],'LEFT')
            ->where($where)
            ->order('create_time','desc')
            ->paginate(10);
        foreach ($list as $row) {
            if (!$row['is_super']) {
                $row['staff'] = Db::table('cb_crm_staff')->where('wid',$wid)->where('id',$row['staff_id'])->field('id,nickname')->find();
            }else{
                $row['staff'] = null;
            }
        }

        return $list;
    }

    /*
     * 添加商机
     * */
    public function addNiche($params,$wid)
    {
        $result = false;
        Db::startTrans();
        try {
            $params['wid'] = $wid;
            $result = $this->save($params);
            (new OperateModel())->addLog("添加客户商机[{$params['title']}]",$params['customer_id'],$wid,$params);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 编辑商机
     * */
    public function editNiche($id,$params,$wid)
    {
        $row = $this->where('id',$id)->where('wid',$wid)->find();
        if (!$row) {
            abort(404,'记录不存在');
        }
        $result = false;
        Db::startTrans();
        try {
            $result = $row->save($params);
            (new OperateModel())->addLog("编辑客户商机[{$params['title']}]",$params['customer_id'],$wid,$params);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }

    /*
     * 删除商机
     * */
    public function delNiche($id,$wid)
    {
        $row = $this->where('id',$id)->where('wid',$wid)->find();
        if (!$row) {
            abort(404,'记录不存在');
        }
        $result = false;
        Db::startTrans();
        try {
            $result = $row->delete();
            (new OperateModel())->addLog("删除客户商机[{$row['title']}]",$row['customer_id'],$wid,$row);
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return $e->getMessage();
        }
        if ($result) {
            return true;
        }
    }


    public function getFormData($wid)
    {
        $stageList = (new DictionaryModel())->getDicValues('商机阶段', $wid);

        return compact('stageList');
    }

    public function getStageTextAttr($data,$row){
        $dictionary = (new DictionaryModel())->getDicValues('商机阶段', $row['id']);
        foreach ($dictionary as $item) {
            if ($row['stage'] == $item['id']) {
                return $item['name'];
            }
        }
        return '';
    }

    public function getStatusTextAttr($data,$row){
        $status = [1=>'进行中',2=>'签单成功',3=>'跟单失败'];
        return isset($status[$row['status']])?$status[$row['status']]:'';
    }

    public function customer()
    {
        return $this->belongsTo(CustomerModel::class,'customer_id','id')->field('id,name');
    }

    public function contact()
    {
        return $this->belongsTo(ContactsModel::class, 'contact_id', 'id')->field('id,name');
    }
}
